@@ -28,6 +28,7 @@
 #include <QDir>
 #include <QListWidget>
 #include <QProgressBar>
+#include <QScrollBar>
 
 #include "mprocess.h"
 
@@ -48,8 +49,14 @@ void MProcess::setupUI(QListWidget *listLog, QProgressBar *progInstall) noexcept
     pal.setColor(QPalette::Base, Qt::black);
     pal.setColor(QPalette::Text, Qt::white);
     listLog->setPalette(pal);
+    QScrollBar *lvscroll = logView->verticalScrollBar();
+    connect(lvscroll, &QScrollBar::rangeChanged, lvscroll, [this, lvscroll](int, int max){
+        if (lvscroll->value() >= prevScrollMax) logView->scrollToBottom();
+        prevScrollMax = max;
+    });
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
 void MProcess::setupChildProcess() noexcept
 {
     if (section && section->rootdir) {
@@ -57,11 +64,12 @@ void MProcess::setupChildProcess() noexcept
         if (chdir("/") != 0) exit(255);
     }
 }
+#endif
 
 inline bool MProcess::checkHalt()
 {
-    if (halting == ThrowHalt) throw("");
-    else if (halting == Halted) return true;
+    if (halting == THROW_HALT) throw("");
+    else if (halting == HALTED) return true;
     return false;
 }
 
@@ -87,13 +95,13 @@ bool MProcess::exec(const QString &program, const QStringList &arguments,
             const QByteArray &StdOut = readAllStandardOutput();
             if (!StdOut.isEmpty()) {
                 qDebug().nospace() << "SOut #" << execount << ": " << StdOut;
-                hasOut = 1;
+                hasOut = true;
             }
         }
         const QByteArray &StdErr = readAllStandardError();
         if (!StdErr.isEmpty()) {
             qDebug().nospace() << "SErr #" << execount << ": " << StdErr;
-            hasOut = 1;
+            hasOut = true;
         }
         if (hasOut) {
             QFont logFont = logEntry->font();
@@ -126,14 +134,14 @@ bool MProcess::exec(const QString &program, const QStringList &arguments, const
     ++execount;
     const QString &cmd = joinCommand(program, arguments);
     qDebug().nospace().noquote() << "Exec #" << execount << ": " << cmd;
-    return exec(program, arguments, input, needRead, log(cmd, LOG_EXEC));
+    return exec(program, arguments, input, needRead, log(cmd, LOG_EXEC, false));
 }
 bool MProcess::shell(const QString &cmd,  const QByteArray *input, bool needRead)
 {
     if (checkHalt()) return false;
     ++execount;
     qDebug().nospace().noquote() << "Bash #" << execount << ": " << cmd;
-    return exec("/usr/bin/bash", {"-c", cmd}, input, needRead, log(cmd, LOG_EXEC));
+    return exec("/usr/bin/bash", {"-c", cmd}, input, needRead, log(cmd, LOG_EXEC, false));
 }
 
 QString MProcess::readOut(bool everything) noexcept
@@ -150,7 +158,7 @@ QStringList MProcess::readOutLines() noexcept
 void MProcess::halt(bool exception) noexcept
 {
     log(__PRETTY_FUNCTION__, LOG_MARKER);
-    halting = exception ? ThrowHalt : Halted;
+    halting = exception ? THROW_HALT : HALTED;
     if(state() != QProcess::NotRunning) {
         QEventLoop eloop;
         connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
@@ -164,7 +172,7 @@ void MProcess::halt(bool exception) noexcept
 }
 void MProcess::unhalt() noexcept
 {
-    if (halting == NoHalt) return;
+    if (halting == NO_HALT) return;
     QEventLoop eloop;
     connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
     if (state() != QProcess::NotRunning) {
@@ -173,7 +181,7 @@ void MProcess::unhalt() noexcept
         closeWriteChannel();
         eloop.exec();
     }
-    halting = NoHalt;
+    halting = NO_HALT;
 }
 
 QString MProcess::joinCommand(const QString &program, const QStringList &arguments) noexcept
@@ -194,12 +202,14 @@ QString MProcess::joinCommand(const QString &program, const QStringList &argumen
     return text;
 }
 
-QListWidgetItem *MProcess::log(const QString &text, const LogType type) noexcept
+QListWidgetItem *MProcess::log(const QString &text, const LogType type, bool save) noexcept
 {
-    if (type == LOG_LOG) qDebug().noquote() << text;
-    else if (type == LOG_MARKER) qDebug().noquote() << "<<" << text << ">>";
-    else if (type == LOG_STATUS) qDebug().noquote() << "++" << text << "++";
-    else if (type == LOG_FAIL) qDebug().noquote() << "--" << text << "--";
+    if (save) {
+        if (type == LOG_MARKER) qDebug().noquote() << "<<" << text << ">>";
+        else if (type == LOG_STATUS) qDebug().noquote() << "++" << text << "++";
+        else if (type == LOG_FAIL) qDebug().noquote() << "--" << text << "--";
+        else qDebug().noquote() << text;
+    }
     if (!logView || type == LOG_MARKER) return nullptr;
     QListWidgetItem *entry = new QListWidgetItem(text, logView);
     logView->addItem(entry);
@@ -210,7 +220,6 @@ QListWidgetItem *MProcess::log(const QString &text, const LogType type) noexcept
         entry->setFont(font);
         if (type == LOG_FAIL) entry->setForeground(Qt::magenta);
     }
-    logView->scrollToBottom();
     return entry;
 }
 void MProcess::log(QListWidgetItem *entry, enum Status status) noexcept
@@ -266,7 +275,7 @@ void MProcess::sleep(const int msec, const bool silent) noexcept
     QListWidgetItem *logEntry = nullptr;
     if (!silent) {
         ++sleepcount;
-        logEntry = log(QString("SLEEP: %1ms").arg(msec), LOG_EXEC);
+        logEntry = log(QString("SLEEP: %1ms").arg(msec), LOG_EXEC, false);
         qDebug().nospace() << "Sleep #" << sleepcount << ": " << msec << "ms";
     }
     QTimer cstimer(this);
@@ -284,7 +293,7 @@ bool MProcess::mkpath(const QString &path, mode_t mode, bool force)
     if (checkHalt()) return false;
     if (!force && QFileInfo::exists(path)) return true;
 
-    QListWidgetItem *logEntry = log("MKPATH: "+path, LOG_EXEC);
+    QListWidgetItem *logEntry = log("MKPATH: "+path, LOG_EXEC, false);
     bool rc = QDir().mkpath(path);
     if (mode && rc) rc = (chmod(path.toUtf8().constData(), mode) == 0);
     qDebug() << (rc ? "MkPath(SUCCESS):" : "MkPath(FAILURE):") << path;
@@ -318,18 +327,6 @@ int MProcess::detectEFI(bool noTest)
     }
     return testEFI;
 }
-bool MProcess::detectMac()
-{
-    bool rc = false;
-    if (testMac < 0) {
-        // if it looks like an apple...
-        rc = exec("grub-probe", {"-d", "/dev/sda2"}, nullptr, true);
-        if (rc) rc = readOut(true).contains("hfsplus");
-        testMac = (rc ? 1 : 0);
-        qDebug() << "Detect Mac:" << rc;
-    }
-    return rc;
-}
 
 // Local execution environment
 
@@ -346,13 +343,30 @@ MProcess::Section::Section(class MProcess &mproc) noexcept
 MProcess::Section::~Section() noexcept
 {
     const char *oldroot = oldsection ? oldsection->rootdir : nullptr;
-    if (qstrcmp(oldroot, rootdir)) proc.log(QString("Revert root: ")+oldroot, LOG_LOG);
+    if (qstrcmp(oldroot, rootdir)) {
+        proc.log(QString("Revert root: ")+oldroot, LOG_LOG);
+    }
     proc.section = oldsection;
 }
 
 // Note: newroot must be valid and unchanged for the life of the chroot.
 void MProcess::Section::setRoot(const char *newroot) noexcept
 {
-    if (qstrcmp(newroot, rootdir)) proc.log(QString("Change root: ")+newroot, LOG_LOG);
+    if (qstrcmp(newroot, rootdir)) {
+        proc.log(QString("Change root: ")+newroot, LOG_LOG);
+    }
     rootdir = newroot; // Might be different pointers to the same text.
+
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+    if (qstrlen(rootdir) > 0) {
+        proc.setChildProcessModifier([this]() {
+            if (rootdir) {
+                if (chroot(rootdir) != 0) _exit(255);
+                if (chdir("/") != 0) _exit(255);
+            }
+        });
+    } else {
+        proc.setChildProcessModifier(nullptr);
+    }
+#endif
 }
